using Arction.WinForms.Charting;
using Arction.WinForms.Charting.Annotations;
using Arction.WinForms.Charting.Axes;
using Arction.WinForms.Charting.SeriesXY;
using Arction.WinForms.Charting.Views.ViewXY;
using System.Text;

namespace WF_dotNET_App
{
    public partial class ExampleCursorTrackingXY : Form
    {
            private LightningChart _chart;

            /// <summary>
            /// Constructor.
            /// </summary>
            public ExampleCursorTrackingXY()
            {
                InitializeComponent();
                CreateChart();
            }

            /// <summary>
            /// Create chart
            /// </summary>
            private void CreateChart()
            {
                //Create new chart
                _chart = new LightningChart();
                _chart.Name = "Cursor tracking chart";

                //Disable rendering, strongly recommended before updating chart properties
                _chart.BeginUpdate();

                //Chart parent must be set
                _chart.Parent = panel1;

                //Fill parent area with chart
                _chart.Dock = DockStyle.Fill;

                //Y-axis are stacked and has common x-axis drawn
                _chart.ViewXY.AxisLayout.YAxesLayout = YAxesLayout.Stacked;

                //Remove existing y-axis
                _chart.ViewXY.YAxes.Clear();

                //Don't show legend box
                _chart.ViewXY.LegendBoxes[0].Visible = false;

                //Chart title text
                _chart.Title.Text = "Wind mill power outputs";

                //X-axis
                _chart.ViewXY.XAxes[0].Title.Text = "Time";
                _chart.ViewXY.XAxes[0].ScrollMode = XAxisScrollMode.None;

                int seriesCount = 8;
                int pointCount = 500;

                Random rand = new Random();
                const double xStep = 0.1;

                //Create series
                for (int i = 0; i < seriesCount; i++)
                {
                    //Create new Y axis for each series 
                    AxisY axisY = new AxisY(_chart.ViewXY);
                    axisY.Units.Text = "kW";
                    axisY.Title.Text = "Mill " + (i + 1).ToString() + "\n[kW]";
                    axisY.Title.Angle = 0;
                    axisY.Title.Font = new Font("Segoe UI", 8f, FontStyle.Regular);
                    axisY.Title.Color = Color.White;
                    axisY.AutoFormatLabels = false;
                    axisY.AutoDivSeparationPercent = 5;
                    axisY.LabelsNumberFormat = "0";

                    _chart.ViewXY.YAxes.Add(axisY);

                    //Create a point-line series
                    PointLineSeries pointLineSeries = new PointLineSeries(_chart.ViewXY, _chart.ViewXY.XAxes[0], axisY);
                    pointLineSeries.LineStyle.Color = DefaultColors.SeriesForBlackBackground[i];

                    //Set some data into series 
                    SeriesPoint[] points = new SeriesPoint[pointCount];
                    double x = 0;
                    for (int point = 0; point < pointCount; point++)
                    {
                        points[point].X = x;
                        points[point].Y = 300.0 + 800.0 * (0.2 * (rand.NextDouble() - 0.5) + 0.2 * Math.Sin(x * (double)(i + 1)));
                        x += xStep;
                    }

                    //Assign the points array to series
                    pointLineSeries.Points = points;

                    //Set title style, for "right-edge" cursor values mode 
                    pointLineSeries.Title.HorizontalAlign = AlignmentHorizontal.Right;
                    pointLineSeries.Title.Fill.Style = RectFillStyle.ColorOnly;
                    pointLineSeries.Title.Fill.Color = ChartTools.CalcGradient(Color.FromArgb(200, Color.White), Color.FromArgb(200, pointLineSeries.LineStyle.Color), 50);
                    pointLineSeries.Title.Fill.GradientColor = ChartTools.CalcGradient(Color.FromArgb(200, Color.Black), Color.FromArgb(200, pointLineSeries.LineStyle.Color), 50);
                    pointLineSeries.Title.Color = Color.White;

                    //Add point-line series to PointLineSeries list
                    _chart.ViewXY.PointLineSeries.Add(pointLineSeries);
                }

                //Add an annotation to show the cursor values
                AnnotationXY cursorValueDisplay = new AnnotationXY(_chart.ViewXY, _chart.ViewXY.XAxes[0], _chart.ViewXY.YAxes[0]);
                cursorValueDisplay.Style = AnnotationStyle.RoundedCallout;
                cursorValueDisplay.LocationCoordinateSystem = CoordinateSystem.RelativeCoordinatesToTarget;
                cursorValueDisplay.LocationRelativeOffset.X = 130;
                cursorValueDisplay.LocationRelativeOffset.Y = -200;
                cursorValueDisplay.Sizing = AnnotationXYSizing.Automatic;
                cursorValueDisplay.TextStyle.Font = new Font("Lucida console", 10f, FontStyle.Regular);
                cursorValueDisplay.TextStyle.Color = Color.Black;
                cursorValueDisplay.Text = "";
                cursorValueDisplay.Fill.Color = Color.White;
                cursorValueDisplay.Fill.GradientColor = Color.FromArgb(120, Color.Cyan);
                cursorValueDisplay.BorderVisible = false;
                cursorValueDisplay.AllowTargetMove = false;
                cursorValueDisplay.Visible = false;
                _chart.ViewXY.Annotations.Add(cursorValueDisplay);

                //Add cursor
                LineSeriesCursor cursor = new LineSeriesCursor(_chart.ViewXY, _chart.ViewXY.XAxes[0]);
                _chart.ViewXY.LineSeriesCursors.Add(cursor);
                cursor.PositionChanged += cursor_PositionChanged;
                cursor.ValueAtXAxis = 10;
                cursor.LineStyle.Color = Color.FromArgb(150, Color.Red);
                cursor.SnapToPoints = checkBoxSnapToNearestDataPoint.Checked;
                cursor.TrackPoint.Color1 = Color.White;

                _chart.ViewXY.ZoomToFit();

                _chart.Load += new EventHandler(_chart_Load);
                _chart.Resize += new EventHandler(_chart_Resize);
                _chart.SizeChanged += new EventHandler(_chart_SizeChanged);

                //Allow chart rendering
                _chart.EndUpdate();

            }

            private void _chart_SizeChanged(object sender, EventArgs e)
            {
                UpdateCursorResult();
            }

            private void cursor_PositionChanged(object sender, PositionChangedEventArgs e)
            {
                // Cancel the on-going rendering because the code below updates 
                // the chart.
                e.CancelRendering = true;

                UpdateCursorResult();
            }

            private void _chart_Load(object sender, EventArgs e)
            {
                UpdateCursorResult();
            }

            /// <summary>
            /// Update cursor texts.
            /// </summary>
            private void UpdateCursorResult()
            {
                bool showNextToCursor = checkBoxShowValuesNextToCursor.Checked;

                //Disable rendering, strongly recommended before updating chart properties
                _chart.BeginUpdate();

                //Get cursor
                LineSeriesCursor cursor = _chart.ViewXY.LineSeriesCursors[0];

                //Get annotation
                AnnotationXY cursorValueDisplay = _chart.ViewXY.Annotations[0];

                //Set annotation target. The location is relative to target. 
                //Use graph bottom as target Y value. 
                float targetYCoord = _chart.ViewXY.GetMarginsRect().Bottom;
                double y;
                _chart.ViewXY.YAxes[0].CoordToValue(targetYCoord, out y);
                cursorValueDisplay.TargetAxisValues.X = cursor.ValueAtXAxis;
                cursorValueDisplay.TargetAxisValues.Y = y;

                double seriesYValue = 0;

                StringBuilder sb = new StringBuilder();
                int seriesIndex = 1;

                string channelStringFormat = "Mill {0}: {1,12:#####.###} {2}";
                bool labelVisible = false;
                bool accurateValue = radioButtonAccurate.Checked;
                string value = "";

                foreach (PointLineSeries pointLineSeries in _chart.ViewXY.PointLineSeries)
                {
                    //show series titles and cursor values in them, on the right side of the chart, 
                    //if cursor values are not shown next to the cursor in an annotation
                    pointLineSeries.Title.Visible = !showNextToCursor;
                    bool resolved = false;
                    value = "";

                    if (accurateValue)
                        resolved = SolveValueAccurate(pointLineSeries, cursor.ValueAtXAxis, out seriesYValue);
                    else
                        resolved = SolveValueCoarse(pointLineSeries, cursor.ValueAtXAxis, out seriesYValue);

                    AxisY axisY = _chart.ViewXY.YAxes[pointLineSeries.AssignYAxisIndex];

                    if (resolved)
                    {
                        labelVisible = true;
                        value = string.Format(channelStringFormat, seriesIndex, seriesYValue.ToString("0.0"), axisY.Units.Text);
                    }
                    else
                    {
                        value = string.Format(channelStringFormat, seriesIndex, "---", axisY.Units.Text);
                    }
                    sb.AppendLine(value);
                    pointLineSeries.Title.Text = value;
                    seriesIndex++;
                }

                sb.AppendLine("");
                sb.AppendLine("Time: " + _chart.ViewXY.XAxes[0].TimeString(cursor.ValueAtXAxis, "HH:mm:ss.ffff"));

                //Set text
                cursorValueDisplay.Text = sb.ToString();

                //Show the label only if it selected to be shown
                cursorValueDisplay.Visible = labelVisible && showNextToCursor;

                //Allow chart rendering
                _chart.EndUpdate();
            }

            private void _chart_Resize(object sender, EventArgs e)
            {
                UpdateCursorResult();
            }

            /// <summary>
            /// Solve value from series data points array. Accurate method, but slower than SolveValueCoarse
            /// </summary>
            /// <param name="series">Series</param>
            /// <param name="xValue">X value</param>
            /// <param name="yValue">Output Y value</param>
            /// <returns>Success status</returns>
            private bool SolveValueAccurate(PointLineSeries series, double xValue, out double yValue)
            {
                AxisY axisY = _chart.ViewXY.YAxes[series.AssignYAxisIndex];
                yValue = 0;

                LineSeriesValueSolveResult result = series.SolveYValueAtXValue(xValue);
                if (result.SolveStatus == LineSeriesSolveStatus.OK)
                {
                    //PointLineSeries may have two or more points at same X value. If so, center it between min and max 
                    yValue = (result.YMax + result.YMin) / 2.0;
                    return true;
                }
                else
                {
                    return false;
                }
            }

            /// <summary>
            /// Solve value from screen coordinates. Faster method, but not less accurateValue than SolveValueAccurate 
            /// </summary>
            /// <param name="series">Series</param>
            /// <param name="xValue">X value</param>
            /// <param name="yValue">Output Y value</param>
            /// <returns>Success status</returns>
            private bool SolveValueCoarse(PointLineSeries series, double xValue, out double yValue)
            {
                AxisY axisY = _chart.ViewXY.YAxes[series.AssignYAxisIndex];
                float coordX = _chart.ViewXY.XAxes[0].ValueToCoord(xValue);
                float coordY;
                yValue = 0;

                LineSeriesCoordinateSolveResult result = series.SolveYCoordAtXCoord(coordX);
                if (result.SolveStatus == LineSeriesSolveStatus.OK)
                {
                    coordY = (result.CoordBottom + result.CoordTop) / 2f;
                    if (axisY.CoordToValue((int)Math.Round(coordY), out yValue) == false)
                    {
                        return false;
                    }
                }
                else
                {
                    return false;
                }

                return true;
            }

            /// <summary>
            /// Toggle value visibility next to cursor.
            /// </summary>
            /// <param name="sender">sender</param>
            /// <param name="e">arguments</param>
            private void checkBoxShowValuesNextToCursor_CheckedChanged(object sender, EventArgs e)
            {
                UpdateCursorResult();
            }

            private void checkBoxSnapToNearestDataPoint_CheckedChanged(object sender, EventArgs e)
            {
                if (_chart != null)
                    _chart.ViewXY.LineSeriesCursors[0].SnapToPoints = checkBoxSnapToNearestDataPoint.Checked;
            }
        }
    }
